#include "hd44780.h"
#include <avr/pgmspace.h>

//----------------------------------------------------------------
void lcdSendNibble (char byte, char state)
{
	LCDCONTROLDDR |= LCD_CONTROL_MASK;
	LCDDATADDR |= LCD_DATA_MASK;
	
	LCDCONTROLPORT &= ~(1<<LCD_RW);
	
	if (state)	
		LCDCONTROLPORT |= (1<<LCD_RS);
	else
		LCDCONTROLPORT &= ~(1<<LCD_RS);
		
	LCDCONTROLPORT |= (1<<LCD_E);
		
	LCDDATAPORT &= ~LCD_DATA_MASK;
	
	if (byte & (1<<3)) LCDDATAPORT |= (1<<LCD_D7);
	if (byte & (1<<2)) LCDDATAPORT |= (1<<LCD_D6);
	if (byte & (1<<1)) LCDDATAPORT |= (1<<LCD_D5);
 	if (byte & (1<<0)) LCDDATAPORT |= (1<<LCD_D4);
	
	_delay_us (LCD_STROBEDELAY_US);
	
	LCDCONTROLPORT &= ~(1<<LCD_E);
}

//----------------------------------------------------------------
char lcdGetNibble (char state)
{
	char temp = 0;
	LCDCONTROLDDR |= LCD_CONTROL_MASK;
	LCDCONTROLPORT |= (1<<LCD_RW);
	
	if (state)						
		LCDCONTROLPORT |= (1<<LCD_RS);
	else
		LCDCONTROLPORT &= ~(1<<LCD_RS);

	LCDCONTROLPORT |=	(1<<LCD_E);

	LCDDATADDR &= ~LCD_DATA_MASK;
	LCDDATAPORT |= LCD_DATA_MASK;
				
	_delay_us (LCD_STROBEDELAY_US);	

	LCDCONTROLPORT &= ~(1<<LCD_E);	
		
	if (LCDDATAPIN & (1<<LCD_D7)) temp |= (1<<3);
	if (LCDDATAPIN & (1<<LCD_D6)) temp |= (1<<2);
	if (LCDDATAPIN & (1<<LCD_D5)) temp |= (1<<1);
 	if (LCDDATAPIN & (1<<LCD_D4)) temp |= (1<<0);
	
	return temp;	
}

//----------------------------------------------------------------
char lcdRawGetByte (char state)
{
	char temp = 0;
	
	temp |= lcdGetNibble (state);
	temp = temp << 4;					
	temp |= lcdGetNibble (state);
	
	return temp;
}

//----------------------------------------------------------------
void lcdRawSendByte (char byte, char state)
{
	#ifdef lcdEnableRussian
		if (state) byte = toRussian (byte);
	#endif

	lcdSendNibble ((byte>>4), state);					
	lcdSendNibble (byte, state);
}

//----------------------------------------------------------------
char lcdIsBusy (void)
{
	if (lcdRawGetByte(LCD_COMMAND) & (1<<7))
		return 0xff;
	else
		return 0x00;
}

//----------------------------------------------------------------
void lcdInit (void)
{
	while (lcdIsBusy());	
	lcdSendNibble(0b0010, LCD_COMMAND);

	while (lcdIsBusy());
	lcdRawSendByte(0b00101000, LCD_COMMAND);

	while (lcdIsBusy());
	lcdRawSendByte(0b00000001, LCD_COMMAND);

	while (lcdIsBusy());
	lcdRawSendByte(0b00000110, LCD_COMMAND);

	while (lcdIsBusy());
	lcdRawSendByte(0b00001100, LCD_COMMAND);
}

//----------------------------------------------------------------
void lcdSetCursor (char cursor)
{
	while (lcdIsBusy())	;
	lcdRawSendByte((0b00001100 | cursor), LCD_COMMAND);
}

//----------------------------------------------------------------
void lcdSetDisplay (char state)
{
	while (lcdIsBusy())	;
	lcdRawSendByte ((0b00001000 | state), LCD_COMMAND);	
}

//----------------------------------------------------------------
void lcdClear (void)
{
	while (lcdIsBusy()) ;	
	lcdRawSendByte (0b00000001, LCD_COMMAND);
}

//----------------------------------------------------------------
void lcdGotoXY (char str, char col)
{
	char offset [4];
	offset [0] = LCD_ROW1_OFFSET;
	offset [1] = LCD_ROW2_OFFSET;
	offset [2] = LCD_ROW3_OFFSET;
	offset [3] = LCD_ROW4_OFFSET;

	while (lcdIsBusy())	;	
	lcdRawSendByte ((0b10000000 | (offset [(unsigned char) str] + col)), LCD_COMMAND);
}

//----------------------------------------------------------------
void lcdDisplayScroll (char pos, char dir)
{
	while (pos)
	{
		while (lcdIsBusy());
		lcdRawSendByte((0b00011000 | dir), LCD_COMMAND);
		pos--;
	}	
}

//----------------------------------------------------------------
void lcdPutsFromRAM (char *str)
{
	while (*str)
	{
		while (lcdIsBusy());
		lcdRawSendByte (*str++, LCD_DATA);
	}
}

//----------------------------------------------------------------
void lcdPutsFromFlash (char *str)
{
	while (pgm_read_byte (str))
	{
		while (lcdIsBusy());	
		lcdRawSendByte (pgm_read_byte(str++), LCD_DATA);
	}
}

//----------------------------------------------------------------
void lcdLoadCharacterFromRAM (char code, char *pattern)
{
	while (lcdIsBusy());
	lcdRawSendByte ((code<<3) | 0b01000000, LCD_COMMAND);
	
	for (char i = 0; i <= 7; i++)
	{
		while (lcdIsBusy());			
		lcdRawSendByte (*pattern++, LCD_DATA);
	}
	while (lcdIsBusy()) ;
	lcdRawSendByte (0b10000000, LCD_COMMAND);
}

//----------------------------------------------------------------
void lcdLoadCharacterFromFlash(char code, char *pattern)
{
	while (lcdIsBusy());
	lcdRawSendByte ((code<<3) | 0b01000000, LCD_COMMAND);
	
	for (char i = 0; i <= 7; i++)
	{
		while (lcdIsBusy());			
		lcdRawSendByte (pgm_read_byte (pattern++), LCD_DATA);
	}
	while (lcdIsBusy()) ;
	lcdRawSendByte (0b10000000, LCD_COMMAND);
}

//----------------------------------------------------------------
#ifdef lcdEnableRussian
char toRussian (char byte)
{
    switch (byte)
    {
            case 0xC0: byte = 0x41; break;
            case 0xC1: byte = 0xA0; break;
            case 0xC2: byte = 0x42; break;
            case 0xC3: byte = 0xA1; break;
            case 0xC4: byte = 0xE0; break;
            case 0xC5: byte = 0x45; break;
            case 0xA8: byte = 0xA2; break;
            case 0xC6: byte = 0xA3; break;
            case 0xC7: byte = 0xA4; break;
            case 0xC8: byte = 0xA5; break;
            case 0xC9: byte = 0xA6; break;
            case 0xCA: byte = 0x4B; break;
            case 0xCB: byte = 0xA7; break;
            case 0xCC: byte = 0x4D; break;
            case 0xCD: byte = 0x48; break;
            case 0xCE: byte = 0x4F; break;
            case 0xCF: byte = 0xA8; break;
            case 0xD0: byte = 0x50; break;
            case 0xD1: byte = 0x43; break;
            case 0xD2: byte = 0x54; break;
            case 0xD3: byte = 0xA9; break;
            case 0xD4: byte = 0xAA; break;
            case 0xD5: byte = 0x58; break;
            case 0xD6: byte = 0xE1; break;
            case 0xD7: byte = 0xAB; break;
            case 0xD8: byte = 0xAC; break;
            case 0xD9: byte = 0xE2; break;
            case 0xDA: byte = 0xAD; break;
            case 0xDB: byte = 0xAE; break;
            case 0xDC: byte = 0x62; break;
            case 0xDD: byte = 0xAF; break;
            case 0xDE: byte = 0xB0; break;
            case 0xDF: byte = 0xB1; break;
            case 0xE0: byte = 0x61; break;
            case 0xE1: byte = 0xB2; break;
            case 0xE2: byte = 0xB3; break;
            case 0xE3: byte = 0xB4; break;
            case 0xE4: byte = 0xE3; break;
            case 0xE5: byte = 0x65; break;
            case 0xB8: byte = 0xB5; break;
            case 0xE6: byte = 0xB6; break;
            case 0xE7: byte = 0xB7; break;
            case 0xE8: byte = 0xB8; break;
            case 0xE9: byte = 0xB9; break;
            case 0xEA: byte = 0xBA; break;
            case 0xEB: byte = 0xBB; break;
            case 0xEC: byte = 0xBC; break;
            case 0xED: byte = 0xBD; break;
            case 0xEE: byte = 0x6F; break;
            case 0xEF: byte = 0xBE; break;
            case 0xF0: byte = 0x70; break;
            case 0xF1: byte = 0x63; break;
            case 0xF2: byte = 0xBF; break;
            case 0xF3: byte = 0x79; break;
            case 0xF4: byte = 0xE4; break;
            case 0xF5: byte = 0x78; break;
            case 0xF6: byte = 0xE5; break;
            case 0xF7: byte = 0xC0; break;
            case 0xF8: byte = 0xC1; break;
            case 0xF9: byte = 0xE6; break;
            case 0xFA: byte = 0xC2; break;
            case 0xFB: byte = 0xC3; break;
            case 0xFC: byte = 0xC4; break;
            case 0xFD: byte = 0xC5; break;
            case 0xFE: byte = 0xC6; break;
            case 0xFF: byte = 0xC7; break;
    }
	return byte;
}
#endif
